#include <stdio.h>
#include "common.h"

/*
// Example of node numbering
// of a 3x3 2D-Torus
//
// 0 - 1 - 2
// |   |   |
// 3 - 4 - 5
// |   |   |
// 6 - 7 - 8
//
*/

extern int DIMY, numRanks ;

#define ROW( p )   ( p / DIMY )
#define COL( p )   ( p % DIMY )
#define DOWN( p )  ( (p + DIMY) % numRanks )
#define UP( p )    ( (p + numRanks - DIMY) % numRanks )
#define RIGHT( p ) ( p + ( ( (p % DIMY) == DIMY-1 ) ? ( 1 - DIMY ) : 1 ) )
#define LEFT( p )  ( p + ( ( (p % DIMY) == 0 ) ? ( DIMY - 1 ) : - 1 ) )

void recv( pkt *p, int myRank )
{
    /* Algorithm:
     * 
     * 1. Check if src is along my x dimension.
     * 2. If so,
     *    a. send along y dimension.
     *    b. Also forward along x dimension if not last node (to left of src).
     * 3. If not,
     *    a. forward along y dimension if not last node (above line of src).
     * 4. consume.
     */

    if ( ROW( p->src ) == ROW( myRank ) )
    {
        send( p, myRank, DOWN(myRank) ) ;
        if ( LEFT(p->src) != myRank )
            send( p, myRank, RIGHT(myRank) ) ;
    }
    else if ( ROW(UP(p->src)) != ROW(myRank) )
            send( p, myRank, DOWN(myRank) ) ;

    consume( p, myRank ) ;
}

void broadcast( pkt *p, int src )
{
   p->src = src ;
   p->dst = -1 ;
   recv( p, src ) ;
}
